/** @file   problem1.c
 	 @brief  Solution to Problem 1, May exam 2012-2013
    @author Juan Gomez Romero
    @date   2012-2013
 */
 
#include <stdio.h>
#include <stdlib.h>

#define PLAYERS 100
#define MAX_STRING 64
#define ROWS 2
#define COLS 3

struct Player {
   char name[64];
   int age;
   int card[ROWS][COLS];
};

/** @fn            void initializeCard(int c[ROWS][COLS])
    @brief         Initialize user game card with random numbers in {1, 2, , 90} 
    @param[in,out] card Matrix representing a card grid */
void initializeCard(int c[ROWS][COLS]);

/** @fn            void printPlayerData(struct Player p)
    @brief         Print player data 
    @param[in]     p Player to print */
void printPlayerData(struct Player p);

/* Program functions (documented and implemented below) */
void initializePlayersData(struct Player p[], int n);
void crossOutNumber(int c[ROWS][COLS], int d);
int testRow(int c[ROWS][COLS], int r);
int testBingo(int c[ROWS][COLS]);
int getNumbersToBingo(int c[ROWS][COLS]);
void showRanking(struct Player p[], int n);

int main(void) {
   int nPlayers;
   int i, called, endGame;
   
   // 1. Define and declare suitable data structures
   struct Player players[PLAYERS];
   do {
      printf("Number of players? ");
      scanf("%i", &nPlayers);
   } while(nPlayers<1 || nPlayers>100);
   
   // 2. initializePlayersData
   initializePlayersData(players, nPlayers);
   
   // --> display data
   for(i=0; i<nPlayers; i++)
      printPlayerData(players[i]);
   
   // 8. Game loop
   printf("\nGame started!");
   endGame = 0;
   do {
      // draw number
      do {
         printf("\nNumber called?  ");
         scanf("%i", &called);
      } while(called<1 || called>90);
      
      // cross out all players' cards
      for(i=0; i<nPlayers; i++) {
         crossOutNumber(players[i].card, called);
         printPlayerData(players[i]);
      }
         
      // check if we have a winner
      for(i=0; i<nPlayers; i++) 
         if(testBingo(players[i].card) == 1) {
            printf("\nPlayer %s calls out bingo!", players[i].name);
            endGame = 1;
         }            
      
   } while(!endGame);
   
   // 7. Ranking
   showRanking(players, nPlayers);
   
   return 0;
}


/************************************************************/
/** @fn            void initializePlayersData(struct Player p[], int n)
    @brief         Function to initialize player data
    @param[in,out] p Players to initialize 
    @param[in]     n Number of players in array p */
void initializePlayersData(struct Player p[], int n) {
   int i;
   for(i=0; i<n; i++) {
      printf("\nInitializing player %i\n", i);
      printf("Name? ");
      scanf("%s", p[i].name);
      printf("Age? ");
      scanf("%i", &p[i].age);
      initializeCard(p[i].card);      
   }
   
   return;
}

/** @fn            void crossOutNumber(int c[ROWS][COLS], int d)
    @brief         Cross out numbers of card c
    @param[in,out] c Card to process
    @param[in]     d Number to cross */
void crossOutNumber(int c[ROWS][COLS], int d) {
   int i, j;
   for(i=0; i<ROWS; i++)
      for(j=0; j<COLS; j++)
         if(c[i][j] == d)
            c[i][j] = 0;
            
   return;
}

/** @fn            int testRow(int c[ROWS][COLS], int r)
    @brief         Check if all the numbers of a row have been crossed (i.e., they are 0)
    @param[in]     c Card to process
    @param[in]     r Number of row to test 
    @return        1 if all the numbers at row r of c have been crossed */
int testRow(int c[ROWS][COLS], int r) {
   int j, allCrossed = 1;
   for(j=0; j<COLS && allCrossed; j++)
      if(c[r][j] != 0)
         allCrossed = 0;
   
   return allCrossed;
}

/** @fn            int testBingo(int c[ROWS][COLS])
    @brief         Check if all the numbers of the card have been crossed (i.e., they are 0)
                   by relying on testRow function
    @param[in]     c Card to process    
    @return        1 if all the numbers of the card have been crossed */
int testBingo(int c[ROWS][COLS]) {
   int i, allCrossed = 1;
   for(i=0; i<ROWS && allCrossed; i++)
      allCrossed &= testRow(c, i);
   
   return allCrossed;
}

/** @fn            int getNumbersToBingo(int c[ROWS][COLS])
    @brief         Calculates how many numbers remain uncrossed in card c
    @param[in]     c Card to process    
    @return        Count of numbers remaining uncrossed in card c */
int getNumbersToBingo(int c[ROWS][COLS]) {
   int notCrossed = 0;
   int i, j;
   for(i=0; i<ROWS; i++)
      for(j=0; j<COLS; j++)
         notCrossed += c[i][j] > 0;
         
   return notCrossed;
}

/** @fn            void showRanking(struct Player p[], int n)
    @brief         Prints on the screen the name of the players and the number of
                   uncrossed numbers in their cards in increasing order
    @param[in]     p Players array
    @param[in]     n Number of elements in p array */
void showRanking(struct Player p[], int n) {
   struct Player p_copy[PLAYERS];
   int toBingo[PLAYERS], i, tempNumber, swapped;
   struct Player tempPlayer;
   
   // copy array and pre-store numbers to bingo
   for(i=0; i<n; i++) {
      p_copy[i]  = p[i];
      toBingo[i] = getNumbersToBingo(p_copy[i].card);
   }
            
   // sort array
   do {
      swapped = 0;
      for(i=0; i<n-1; i++) {
         if(toBingo[i] > toBingo[i+1]) {
            // swap players
            tempPlayer = p_copy[i];
            p_copy[i]  = p_copy[i+1];
            p_copy[i+1]= tempPlayer;
            
            // swap toBingo values
            tempNumber   = toBingo[i];
            toBingo[i]   = toBingo[i+1];
            toBingo[i+1] = tempNumber;
            
            swapped = 1; 
         }
      }
      
   } while(swapped);
   
   // display ranking
   printf("\n\nRanking\n");
   for(i=0; i<n; i++)
      printf("%40s: %2i remaining \n", p_copy[i].name, toBingo[i]);

   return;
}

/************************************************************/
void initializeCard(int c[ROWS][COLS]) {
   int i, j;
   for(i=0; i<ROWS; i++)
      for(j=0; j<COLS; j++)
         c[i][j] = 1 + rand() % 90;
   
   return;
}

void printPlayerData(struct Player p) {
   int i, j;
   printf("\nPlayer %s (%i) \n", p.name, p.age);
   for(i=0; i<ROWS; i++) {
      for(j=0; j<COLS; j++)
         printf("%2i ", p.card[i][j]);
      printf("\n");
   }
   
   return;      
}
