/** @file   problem2.c
 	 @brief  Solution to Problem 2, May exam 2012-2013
    @author Juan Gomez Romero
    @date   2012-2013
 */

#include <stdio.h>
#include <math.h>

#define TANKS 4
#define MAX_STRING 64

#define FULL        2
#define HALF_FILLED 1
#define EMPTY       0

#define HIGH        2
#define NORMAL      1
#define NONE        0

// 1. Structure declaration
struct Tank {
   char tankName[MAX_STRING];
   int  stored;
   char districtName[MAX_STRING];
   int  demand;
   int  stress;
};

// 2. Program implementation
int main(void) {
   int i, option, exit, demandRead, allOk;
   int actionTank, sourceTank, destinationTank, availableInSource, remainingInDestination, transfer;
   
   struct Tank tanks[4] = {            // using -1 to represent non-initialized values
      {"Alpha", 2, "North", -1, -1}, 
      {"Bravo", 0, "South", -1, -1}, 
      {"Charlie", 1, "East", -1, -1}, 
      {"Bravo", 1, "West", -1, -1}, 
   };
   demandRead = 0;
   allOk = 0; 
   exit = 0; 
        
   do {
      // print menu
      printf("\n\n");
      printf("\n1. Inspect the amount of water stored by each tank");
      printf("\n2. Read demand values of all districts");
      printf("\n3. Fill up a tank");
      printf("\n4. Drain 1/2 of a tank");
      printf("\n5. Empty a tank");
      printf("\n6. Transfer water from one tank to another");
      printf("\n7. Display recommended actions");
      printf("\n8. Exit");
      printf("\n? ");
      scanf("%i", &option);

      switch(option) {
         case 1: // inspect
            printf("\nInspecting water storage...");
            for(i=0; i<TANKS; i++)
               printf("\nTank %s (%s District): %i", tanks[i].tankName, tanks[i].districtName, tanks[i].stored);
            break;
         
         case 2: // read values
            printf("\nReading demand...\n");
            for(i=0; i<TANKS; i++) {               
               do {               
                  printf("Enter demand of District %s (%i, %i, %i): ", tanks[i].districtName, NONE, NORMAL, HIGH);
                  scanf("%i", &tanks[i].demand);
               } while(tanks[i].demand<NONE || tanks[i].demand>HIGH);               
            }
            demandRead = 1;
            break;
         
         case 3: // fill up tank            
            do {
               printf("\nEnter tank to fill [0, %i]: ", TANKS-1);
               scanf("%i", &actionTank);
            } while(actionTank<0 || actionTank>=TANKS);
            if(tanks[actionTank].stored == FULL) {
               printf("Tank %s is full. Cannot fill a full tank.", tanks[actionTank].tankName);
            } else {
               tanks[actionTank].stored = FULL;
               printf("Tank %s is now full.", tanks[actionTank].tankName);
            }
            break;
         
         case 4: // drain 1/2            
            do {
               printf("\nEnter tank to drain [0, %i]: ", TANKS-1);
               scanf("%i", &actionTank);
            } while(actionTank<0 || actionTank>=TANKS);
            if(tanks[actionTank].stored == EMPTY) {         
               printf("Tank %s is empty. Cannot drain an empty tank.", tanks[actionTank].tankName);               
            } else {
               tanks[actionTank].stored -= 1;
               printf("Tank %s has been partially drained.", tanks[actionTank].tankName);
            }
            break;
            
         case 5: // empty            
            do {
               printf("\nEnter tank to empty [0, %i]: ", TANKS-1);
               scanf("%i", &actionTank);
            } while(actionTank<0 || actionTank>=TANKS);
            if(tanks[actionTank].stored == EMPTY) {         
               printf("Tank %s is empty. Cannot empty an empty tank.", tanks[actionTank].tankName);                           
            } else {
               tanks[actionTank].stored = EMPTY;
               printf("Tank %s is now empty.", tanks[actionTank].tankName);
            }
            break;
            
         case 6: // transfer water            
            printf("\n");
            do {
               printf("Enter source tank [0, %i]: ", TANKS-1);
               scanf("%i", &sourceTank);
            } while(sourceTank<0 || sourceTank>=TANKS);
            
            do {
               printf("Enter destination tank [0, %i] (excluded %i): ", TANKS-1, sourceTank);
               scanf("%i", &destinationTank);
            } while(destinationTank<0 || destinationTank>=TANKS || sourceTank==destinationTank);
            
            printf("Transferring from %i to %i...", sourceTank, destinationTank);                        
            availableInSource      = tanks[sourceTank].stored;
            remainingInDestination = FULL-tanks[destinationTank].stored;
            
            if(availableInSource == 0)
               printf("\nTank %s is empty. Cannot transfer from an empty tank.", tanks[sourceTank].tankName);
            else if(remainingInDestination == 0)
               printf("\nTank %s is full. Cannot transfer to a full tank.", tanks[destinationTank].tankName);            
            else {            
               transfer = availableInSource<remainingInDestination?availableInSource:remainingInDestination;
               tanks[sourceTank].stored      -= transfer;            
               tanks[destinationTank].stored += transfer;
            }
            break;
            
         case 7: // display recommended actions
            if(demandRead) {            
               printf("\nRecommended actions...");
               for(i=0; i<TANKS; i++)
                  if(tanks[i].stress < 0)
                     printf("\nDistrict %s is suffering a water shortage (stress: %i)", tanks[i].districtName, tanks[i].stress);
            } else {
               printf("\nDemand has not been read yet.");
            }
            break;
            
         case 8: // exit
            exit = 1;
            break;
            
         default: // wrong option
            printf("Wrong option\n");
            break;              
      }
      
      if(demandRead) {      
         // update stress values
         for(i=0; i<TANKS; i++)
            tanks[i].stress = tanks[i].stored - tanks[i].demand;
         
         // check if all districts are OK        
         allOk = 1;
         for(i=0; i<TANKS && allOk; i++)
            allOk &= tanks[i].stress >= 0;
      }
      
   } while( !exit && !allOk);
   
   if(exit && !allOk) {
      printf("\n\nUser selected exit, pending recommendations.");
   } else if(exit) {
      printf("\n\nUser selected exit, not pending recommendations.");
   } else {
      printf("\n\nNot pending recommendations, water supply is OK.");
   }
   
   return 0;
}
